#include "gui_functions.h"
#include "xine_iface_gui.h"
#include "flxine.h"
#include <FL/filename.H>
#include <dirent.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <xine.h>

char full_path[XINE_PATH_MAX + XINE_NAME_MAX + 1];
char filename[XINE_NAME_MAX];

int
load_dir ()
{
	dirent **list;
	int num_files, file_ok, file_isdir;
	char *media_extensions;
	media_extensions = new char[strlen (xinei->get_extensions ()) + 1];
	strcpy (media_extensions, xinei->get_extensions ());
	/*fprintf(stderr, "Valid media extensions: %s\n",media_extensions);
	 * gui_file_filter->tooltip(strcat(
	 * "This will enable enable a file filter, and only display files ending in: ",
	 * media_extensions)); */
	fl_filename_absolute (cur_dir, (const char *) cur_dir);
	gui_filename_input->value (cur_dir);

	Fl_File_Sort_F *sort = fl_casealphasort;
	num_files = fl_filename_list (cur_dir, &list, sort);

	gui_file_browser->clear ();
	if (strcmp (cur_dir, "/"))
		gui_file_browser->add ("../");
	for (int dirs = 1; dirs >= 0; dirs--)	//Used to put directories first
		for (int i = 0; i < num_files; i++)
		{
			file_ok = 1;
			file_isdir = 0;
			sprintf (full_path, "%s%s", cur_dir, list[i]->d_name);
			/*Stick a / at the end of the entry if it is a directory */
			if (fl_filename_isdir (full_path))
			{
				sprintf (filename, "%s/", list[i]->d_name);
				file_isdir = 1;
				if (!dirs)
					file_ok = 0;
			}
			else if (dirs)
				file_ok = 0;
			else
				strcpy (filename, list[i]->d_name);
			/*Filter out all the files we don't want to see */
			if (file_ok && filename[0] == '.')
			{
				if (gui_file_hidden->value ()
				    && filename[1] != '.'
				    && filename[1] != '/')
				{
				}
				else
					file_ok = 0;
			}
			if (file_ok && gui_file_filter->value ()
			    && !file_isdir)
			{
				char *test_ext;
				test_ext = new char[256];
				strcpy (test_ext,
					fl_filename_ext ((const char *)
							 filename));
				if (test_ext[0] == '.')
					test_ext++;
				if (strlen (test_ext) > 2)
				{
					for (int j = 0; test_ext[j] != '\0';
					     j++)
						test_ext[j] =
							_tolower (test_ext
								  [j]);
					if (strstr
					    (media_extensions,
					     test_ext) != NULL)
						gui_file_browser->
							add (filename);
				}
			}
			else if (file_ok
				 && (!gui_file_filter->value ()
				     || file_isdir))
				gui_file_browser->add (filename);
		}
	return 0;
}

int
select_file (const char *file)
{
	/* First test to see if we selected a directory,
	 * and if so we are changing to it               */
	char *abs_path, *raw_path;
	int good_sel;
	abs_path = new char[XINE_PATH_MAX];
	raw_path = new char[XINE_PATH_MAX];

	if (strcmp (file, "../") == 0)
	{
		good_sel = up_dir ();
		load_dir ();
		return good_sel;
	}
	else if (file[0] == '/')
	{
		cur_dir = "";
	}


	sprintf (raw_path, "%s%s", cur_dir, file);
	if (fl_filename_isdir (raw_path))
	{
		if (raw_path[strlen (raw_path) - 1] != '/')
			raw_path = strcat (raw_path, "/");
		cur_dir = raw_path;
		load_dir ();
	}
	else
	{
		fprintf (stderr, "File: %s was selected\n", raw_path);
		gui_plist_browser->clear ();
		gui_plist_browser->add (raw_path);
		current_item = 0;
		xinei->open (raw_path);
		xinei->play ();
	}

	return 0;
}

int
up_dir ()
{
	int i;
	if (strcmp (cur_dir, "/") != 0)
	{
		/*I'm sure there is a better way to knock off the last directory... */
		char *name;
		char *basename;
		name = strrchr (cur_dir, '/');
		name[0] = '\0';
		basename = strrchr (cur_dir, '/') + 1;
		basename[0] = '\0';
	}
	return 0;
}

void
gui_load_plugins ()
{
	plugin_button_0->label (xinei->get_auto_plugin (0));
	plugin_button_0->tooltip (xinei->get_auto_plugin_desc (0));
	plist_plugin_button_0->label (xinei->get_auto_plugin (0));
	plist_plugin_button_0->tooltip (xinei->get_auto_plugin_desc (0));

	plugin_button_1->label (xinei->get_auto_plugin (1));
	plugin_button_1->tooltip (xinei->get_auto_plugin_desc (1));
	plist_plugin_button_1->label (xinei->get_auto_plugin (1));
	plist_plugin_button_1->tooltip (xinei->get_auto_plugin_desc (1));

	plugin_button_2->label (xinei->get_auto_plugin (2));
	plugin_button_2->tooltip (xinei->get_auto_plugin_desc (2));
	plist_plugin_button_2->label (xinei->get_auto_plugin (2));
	plist_plugin_button_2->tooltip (xinei->get_auto_plugin_desc (2));

	plugin_button_3->label (xinei->get_auto_plugin (3));
	plugin_button_3->tooltip (xinei->get_auto_plugin_desc (3));
	plist_plugin_button_3->label (xinei->get_auto_plugin (3));
	plist_plugin_button_3->tooltip (xinei->get_auto_plugin_desc (3));

	plugin_button_4->label (xinei->get_auto_plugin (4));
	plugin_button_4->tooltip (xinei->get_auto_plugin_desc (4));
	plist_plugin_button_4->label (xinei->get_auto_plugin (4));
	plist_plugin_button_4->tooltip (xinei->get_auto_plugin_desc (4));

	plugin_button_5->label (xinei->get_auto_plugin (5));
	plugin_button_5->tooltip (xinei->get_auto_plugin_desc (5));
	plist_plugin_button_5->label (xinei->get_auto_plugin (5));
	plist_plugin_button_5->tooltip (xinei->get_auto_plugin_desc (5));

	plugin_button_6->label (xinei->get_auto_plugin (6));
	plugin_button_6->tooltip (xinei->get_auto_plugin_desc (6));
	plist_plugin_button_6->label (xinei->get_auto_plugin (6));
	plist_plugin_button_6->tooltip (xinei->get_auto_plugin_desc (6));

	int i, current = 0;

	const char *const *audio_drivers = xinei->get_audio_drivers ();

	gui_audio_driver_choice->add ("auto");
	for (i = 0; audio_drivers[i] != NULL; i++)
	{
		gui_audio_driver_choice->add (audio_drivers[i]);
		if (xinei->get_current_audio_driver () == NULL)
			gui_audio_driver_choice->value (0);
		else if (strcmp
			 (audio_drivers[i],
			  xinei->get_current_audio_driver ()) == 0)
			gui_audio_driver_choice->value (i);
	}

	const char *const *video_drivers = xinei->get_video_drivers ();
	gui_vid_driver_choice->add ("auto");
	for (i = 0; video_drivers[i] != NULL; i++)
	{
		gui_vid_driver_choice->add (video_drivers[i]);
		if (xinei->get_current_video_driver () == NULL)
			gui_vid_driver_choice->value (0);
		else if (strcmp
			 (video_drivers[i],
			  xinei->get_current_video_driver ()) == 0)
			gui_vid_driver_choice->value (i);
	}

	const char *const *post_plugins =
		xinei->get_post_plugins (XINE_POST_TYPE_AUDIO_VISUALIZATION);
	for (i = 0; post_plugins[i] != NULL; i++)
		gui_vis_plugin_choice->add (post_plugins[i]);
		
	gui_vid_deinterlace->add("none");
	gui_vid_deinterlace->add("bob");
	gui_vid_deinterlace->add("weave");
	gui_vid_deinterlace->add("greedy");
	gui_vid_deinterlace->add("onefield");
	gui_vid_deinterlace->add("onefield_xv");
	gui_vid_deinterlace->add("linearblend");
}

void
gui_apply_config ()
{
	int reinit_xine=0;
	
	xine_cfg_entry_t vis_entry;
	xinei->config_lookup("flxine.vis_post_plugin",&vis_entry);
	vis_entry.str_value = (char *)gui_vis_plugin_choice->text();
	xinei->config_update(&vis_entry);
	
	xinei->config_lookup("flxine.vis_enable",&vis_entry);
	vis_entry.num_value = gui_vis_plugin_enable->value();
	xinei->config_update(&vis_entry);
	
	if (gui_vis_plugin_enable->value ())
	{
		if (gui_vis_plugin_choice->text ()
		    && strcmp (xinei->get_vis (),
			       gui_vis_plugin_choice->text ()))
		{
			gui_vis_plugin_choice->clear_changed ();
			xinei->use_vis (gui_vis_plugin_enable->value (),
					gui_vis_plugin_choice->text ());
		}
	}
	else
		xinei->use_vis (false, NULL);

	if (gui_audio_driver_choice->changed())
	{
		xine_cfg_entry_t audio_entry;
		xinei->config_lookup ("flxine.audio_driver", &audio_entry);
		audio_entry.str_value = (char *)gui_audio_driver_choice->text();
		xinei->config_update (&audio_entry);	
		gui_audio_driver_choice->clear_changed();
		fprintf (stdout, "setting audio driver to: %s\n",
			 gui_audio_driver_choice->text ());
		reinit_xine = 1;
	}
	
	if (gui_vid_driver_choice->changed())
	{
		xine_cfg_entry_t video_entry;
		xinei->config_lookup ("flxine.video_driver", &video_entry);
		video_entry.str_value = (char *)gui_vid_driver_choice->text();
		xinei->config_update (&video_entry);	
		gui_vid_driver_choice->clear_changed();
		fprintf (stdout, "setting video driver to: %s\n",
			 gui_vid_driver_choice->text ());
		reinit_xine = 1;
	}
	
	if(reinit_xine)
		refresh_drivers();
}



void
refresh_drivers ()
{
	int pos, time, len, status, speed;
	char *old_mrl = (char *)xinei->mrl();
	if(xinei->mrl() != NULL)
	{
		speed = xinei->get_speed();
		xinei->get_stream_time(&pos,&time, &len);		
	}
	xinei->stop ();
	xinei->deinit ();
	xinei->init(gui_vid_driver_choice->text(),gui_audio_driver_choice->text());
	if(old_mrl != NULL)
		xinei->open(old_mrl);
	
	if(old_mrl != NULL)
	{
		xinei->play();
		xinei->set_position(pos);
		xinei->set_speed(speed);
	}	
}

void
load_settings ()
{
	int i;
	xine_cfg_entry_t entry;
	if ( xinei->config_lookup ("flxine.video_driver", &entry) ==0 )
		xinei->config_new_string ("flxine.video_driver",
					  "auto",
					  "Video Driver",
					  "Video driver that xine engine should use",
					  20, NULL, NULL);
	xinei->config_lookup ("flxine.video_driver", &entry);
	for(i=0; i<gui_vid_driver_choice->size()-1; i++)
		if( strcmp(gui_vid_driver_choice->text(i),entry.str_value)==0 )
			gui_vid_driver_choice->value(i);
	
	if (!xinei->config_lookup ("flxine.audio_driver", &entry))
		xinei->config_new_string ("flxine.audio_driver",
					  "auto",
					  "Audio Driver",
					  "Audio driver that xine engine should use",
					  20, NULL, NULL);
	xinei->config_lookup ("flxine.audio_driver", &entry);
	
	for(i=0; i<gui_audio_driver_choice->size()-1; i++)		
		if( strcmp(gui_audio_driver_choice->text(i),entry.str_value)==0)			
			gui_audio_driver_choice->value(i);
	
	if (!xinei->config_lookup ("flxine.vis_post_plugin", &entry))
		xinei->config_new_string ("flxine.vis_post_plugin",
					  "",
					  "Visualization Plugin",
					  "Plugin to use when some visualization is desired for streams with no video",
					  10, NULL, NULL);
	xinei->config_lookup ("flxine.vis_post_plugin", &entry);
	for(i=0; i<gui_vis_plugin_choice->size()-1; i++)		
		if( strcmp(gui_vis_plugin_choice->text(i),entry.str_value)==0)			
			gui_vis_plugin_choice->value(i);
	
	if (!xinei->config_lookup ("flxine.vis_enable", &entry))
		xinei->config_new_bool("flxine.vis_enable",
					  0,
					  "Enable Visualization Plugin",
					  "Do you want to display a visualization where there is no video to display?",
					  10, NULL, NULL);
	
	
	//xinei->config_lookup ("flxine.vis_enable", &entry);
	//gui_vis_plugin_enable->value(entry.num_value);
	
	//xinei->config_lookup("video.deinterlace_method",&entry);
	//gui_vid_deinterlace->value(entry.num_value);
	//fprintf(stderr,"Deinterlace method from config: %i %i\n",entry.type, entry.num_value);	
}

void
save_settings ()
{
	xinei->config_save ();
}


void
plist_add_files ()
{
	char new_file[XINE_PATH_MAX];
	for (int i = 0; i < gui_file_browser->size (); i++)
	{
		if (gui_file_browser->selected (i))
		{
			sprintf (new_file, "%s%s", cur_dir,
				 gui_file_browser->text (i));
			if (!fl_filename_isdir (new_file))
				gui_plist_browser->add (new_file);
		}
	}
}
void
plist_remove_selected ()
{
	for (int i = 0; i <= gui_plist_browser->size (); i++)
		if (gui_plist_browser->selected (i))
		{
			gui_plist_browser->remove (i);
			if (current_item >= i)
				current_item--;
			i--;
		}
}
void
plist_move_up ()
{

	int position = gui_plist_browser->value ();
	if (position == 1)
	{
		gui_plist_browser->move (gui_plist_browser->size (),
					 position);
		position = gui_plist_browser->size ();
	}
	else
		gui_plist_browser->move (position, position--);
	gui_plist_browser->deselect ();
	gui_plist_browser->select (position);
}

void
plist_move_down ()
{
	int position = gui_plist_browser->value ();
	if (position == gui_plist_browser->size ())
	{
		gui_plist_browser->move (1, position);
		position = 1;
	}
	else
		gui_plist_browser->move (position, position++);
	gui_plist_browser->deselect ();
	gui_plist_browser->select (position);
}

void
plist_prev ()
{
	if (gui_plist_browser->size ())
	{
		if (current_item == 1)
			current_item = gui_plist_browser->size ();
		else
			current_item--;
		gui_plist_browser->deselect ();
		gui_plist_browser->value (current_item);
		xinei->open ((char *) gui_plist_browser->text (current_item));
		xinei->play ();
	}
}

void
plist_next ()
{
	if (gui_plist_browser->size ())
	{
		if (current_item == gui_plist_browser->size ())
			current_item = 1;
		else
			current_item++;
		gui_plist_browser->deselect ();
		gui_plist_browser->value (current_item);
		xinei->open ((char *) gui_plist_browser->text (current_item));
		xinei->play ();
	}
}

void
plist_add_plugin (int id)
{
	char **mrls = xinei->get_plugin_autoplay_mrls (id);
	if (mrls != NULL)
		for (int i = 0; mrls[i] != NULL; i++)
			gui_plist_browser->add (mrls[0]);
}

void
toggle_fullscreen ()
{
	timeval tp;
	gettimeofday (&tp, NULL);

	if (!fullscreen_mode && (toggle_fullscreen_time != tp.tv_sec))
	{
		fprintf (stderr, "Entering fullscreen mode\n");
		toggle_fullscreen_time = tp.tv_sec;
		video_restore[0] = video_window->x ();
		video_restore[1] = video_window->y ();
		video_restore[2] = video_window->w ();
		video_restore[3] = video_window->h ();
		video_window->fullscreen ();
		fullscreen_mode = 1;
	}
	else if ((toggle_fullscreen_time != tp.tv_sec) && fullscreen_mode)
	{
		toggle_fullscreen_time = tp.tv_sec;
		fprintf (stderr, "Leaving fullscreen mode\n");
		
		video_window->fullscreen_off (video_restore[0],
					      video_restore[1],
					      video_restore[2],
					      video_restore[3]);
		//video_window->iconize();
		video_window->hide();
		video_window->show ();
		xinei->send_vo_event(XINE_GUI_SEND_DRAWABLE_CHANGED,
							 (void *)fl_xid(video_window));
		fullscreen_mode = 0;
	}
}
