#include <stdlib.h>
#include <sys/time.h>
#include <stdio.h>
#include <sys/wait.h>
#include <stdarg.h>
#include <string.h>

#include <Fl/x.h>
#include <FL/fl_draw.H> // To change the cursor in fullscreen mode
#include <xine.h>

#include "flxine.h"
#include "xine_iface_gui.h"
#include "gui_functions.h"
#include "xine_interface.h"
 
xine_interface *xinei;
char *cur_dir, *cur_mrl;
int fullscreen_mode, current_item;
int stream_pos, stream_len, old_mouse_x, old_mouse_y;
int control_state, plist_state, file_state, last_toggle;
long click_time, mouse_activity;
int toggle_gui_time, toggle_fullscreen_time, pointer_hidden;

static int handle_completion(int unhandled_event)
{
	if( fl_xevent->type == xinei->get_completion_event_id() )
	{
      XEvent completion_event = *fl_xevent;
      xinei->send_vo_event(XINE_GUI_SEND_COMPLETION_EVENT, &completion_event);
      return 1;
	}
//    fprintf(stderr,"Unhandled fltk event with id #:%i, X type:%i\n", unhandled_event,fl_xevent->type);
    return 0;    
}

static void video_mouse_events(void *foo)
{
	if(video_overlay == Fl::belowmouse())
	{	
		timeval tp;
		gettimeofday(&tp, NULL);	
		
		if( (old_mouse_x != Fl::event_x())
			|| (old_mouse_y != Fl::event_y())
			|| Fl::event_button1())
		{
			old_mouse_x = Fl::event_x();
			old_mouse_y = Fl::event_y();
			int button1;
			if(Fl::event_button1() && ( abs(click_time-tp.tv_usec)>125000 ) )
			{
				click_time = tp.tv_usec;
				button1 = 1;
			}
			else
				button1 = 0;
				
			xinei->send_mouse_event(old_mouse_x,
									old_mouse_y,
									button1);			
			mouse_activity = tp.tv_sec;
			fl_cursor(FL_CURSOR_DEFAULT,FL_WHITE,FL_BLACK);
		}
		/*Toggle gui control visibility  */
		if( Fl::event_button3() && ( abs(toggle_gui_time-tp.tv_usec)>250000 ) )
		{
			toggle_gui_time = tp.tv_usec;
			if( gui_window->visible() && !fullscreen_mode )
			{
				gui_window->hide();
				control_state = gui_config_window->visible();
				gui_config_window->hide();
				
				plist_state = gui_plist_window->visible();
				gui_plist_window->hide();
				
				file_state = gui_file_window->visible();
				gui_file_window->hide();
			}
			else if( !fullscreen_mode )
			{	
				gui_window->show();
				if(control_state)
					gui_config_window->show();
				if(plist_state)
					gui_plist_window->show();
				if(file_state)
					gui_file_window->show();				
			}
			/*
			if(fullscreen_mode && full_playback_buttons->visible() )
			{
				full_playback_buttons->hide();
				full_navigation_buttons->hide();
				xinei->send_vo_event(XINE_GUI_SEND_VIDEOWIN_VISIBLE,(void *)fullscreen_mode);
			}
			else if(fullscreen_mode)
			{
				full_playback_buttons->show();
				full_navigation_buttons->show();
				xinei->send_vo_event(XINE_GUI_SEND_VIDEOWIN_VISIBLE,(void *)fullscreen_mode);
			}
			*/
			
		}	
		
		if( abs(tp.tv_sec-mouse_activity)>3 )
		{	
			if(fullscreen_mode)			
				fl_cursor(FL_CURSOR_NONE,FL_WHITE,FL_BLACK);
			
		}
	}
}

static void ui_events(void *foo)
{
	int event_id;
	//void *event_data;
	char *ui_data;
	ui_data = new char[512];
	if( xinei->get_ui_event(&event_id, ui_data) )
	{
		if(event_id == XINE_EVENT_UI_SET_TITLE)
		{
			//ui_data = (xine_ui_data_t *)event_data;
			fprintf(stderr,
					"New title change request from xine stream to %s\n",
					ui_data);
			now_playing->label(ui_data);
		}
		else if(event_id == XINE_EVENT_UI_PLAYBACK_FINISHED)
			playlist_clicker();
	}
}

static void gui_update_omatic(void *foo)
{
	int video_win_w, video_win_h;
	int stream_cpos, stream_time, stream_finish;	
 	usleep(1);
  	video_win_w = video_overlay->w();
	video_win_h = video_overlay->h();
	xinei->set_video_attr( video_window->x(),video_window->y(),video_overlay->x(), video_overlay->y(), &video_win_w, &video_win_h);
	if( (gui_window->visible()) && (xinei->status()>1) )
	{
		xinei->get_stream_time(&stream_cpos, &stream_time, &stream_finish);
		if( (stream_pos+500 < stream_pos) || (stream_time > stream_pos+500) || stream_len !=stream_finish )
		{	stream_pos = stream_cpos;
			if(stream_len != stream_finish)
			{
				if( strcmp(xinei->mrl(),cur_mrl) )
				{
					strcpy(cur_mrl,xinei->mrl());
					now_playing->label(xinei->title());
				}
				stream_len = stream_finish;				
				gui_progress_bar->redraw();
				
			}
			gui_progress_bar->value(stream_pos);
		}
	}
	/*
	if(!fullscreen_mode && full_playback_buttons->visible() )
	{
		full_playback_buttons->hide();
		full_navigation_buttons->hide();
	}
	*/
}

void playlist_clicker()
{
	if(current_item < gui_plist_browser->size())
		plist_next();				
}

int main()
{
	cur_dir = new char[XINE_PATH_MAX];
	cur_mrl = new char[XINE_PATH_MAX];
	control_state=
	plist_state=
	file_state=
	click_time=
	current_item=
	last_toggle =0;
	strcpy(cur_dir,"");
	strcpy(cur_mrl,"");
	stream_len=stream_pos=0;
    XInitThreads();
	cur_dir = new char[XINE_PATH_MAX];
	setenv("DVDCSS_CACHE","/tmp",1);
    xinei = new xine_interface();
	
	init_gui();	
	gui_load_plugins();
	video_window->show();
    gui_window->show();	
	
	load_settings();    
	
    xinei->set_parent(fl_xid(video_window),fl_display,fl_screen);
    xinei->init(gui_vid_driver_choice->text(),gui_audio_driver_choice->text());
	if(gui_vis_plugin_enable->value () && gui_vis_plugin_choice->text())
	{
		xinei->use_vis (gui_vis_plugin_enable->value (),
					gui_vis_plugin_choice->text ());
	}
    
    Fl::add_handler(handle_completion);
	
    Fl::add_idle(ui_events);
	Fl::add_idle(gui_update_omatic);
	Fl::add_idle(video_mouse_events);
	
    while(video_window->shown())
		Fl::wait();
    
	save_settings();
	fprintf(stderr,"Shutting down...\n");
    xinei->deinit();
	xinei->~xine_interface();
	
  return 0;
}
