#ifndef _CLASS_XINE_INTERFACE_
#define _CLASS_XINE_INTERFACE_

#include <xine.h>

#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <X11/Xresource.h>


#include "xine_info.h"
typedef struct plugin_info
{
	char *plugin_id;
	char *plugin_desc;
};

class xine_interface
{
	private:
		Window parent_window;
		xine_info * xine_data;
		char *config_file;
		int num_plugins;
		plugin_info auto_plugins[6];
		char *display_name;
		bool initialized;
		Display *parent_display;
		int parent_screen;
		void send_simple_event (int);
		void osd_init ();
		void osd_deinit ();
		bool start_stream(int);
		int events_pending;
		char *ui_event_data;
		int ui_event_type;

    public:
		xine_interface ();
		~xine_interface ();
		int get_completion_event_id ();
		int send_vo_event (int, void *);
		int send_mouse_event(int,int,int);
		void set_parent (Window, Display *, int);
		void set_video_attr (int, int, int, int, int *, int *);	//win_x, win_y, video_x, video_y, w, h
		bool init (const char *, const char *);
		void deinit ();
	//Events
		bool stream_finished;
		bool new_ui_data;
		void set_ui_event(int, char *);
		bool get_ui_event(int *, char *);
	//User Information data
		//char *Title	
	//Plugin controls
		const char *const *get_video_drivers ();
		char *get_current_video_driver();
		const char *const *get_audio_drivers ();
		char *get_current_audio_driver();
		const char *const *get_post_plugins (int);
		void start_plugin (int);
		char *get_auto_plugin (int);
		char *get_auto_plugin_desc (int);
		char **get_plugin_autoplay_mrls(int);
		char *get_extensions ();
		int use_vis (bool, const char *);
		char *get_vis();
	//Media loading and play control
		int open (char *);
		bool play ();
		void stop ();
		int eject();
		void faster ();
		void slower ();
		void set_speed (int);
		int set_position (int);
		int get_speed ();
		int get_stream_time (int *, int *, int *);
	//Volume Control
		int get_volume ();
		void set_volume (int);
		int volume_up ();
		int volume_down ();
	//Navigation functions
		bool next ();
		bool prev ();
		void nav_up ();
		void nav_down ();
		void nav_left ();
		void nav_right ();
		void nav_select ();
		void nav_topmenu ();
		void nav_menu1 ();
		void nav_menu2 ();
	//stream information
		int status ();
		bool has_video ();
		bool has_audio ();
		uint32_t audio_bitrate ();
		uint32_t audio_samplerate ();
	//Current media information
		const char *mrl ();
		const char *title ();
		const char *artist ();
		const char *album ();
		const char *genre ();
		const char *comment ();
	//OSD Controls
		void osd_update ();
		void osd_stream_infos ();
		void osd_draw_bar (char *, int, int, int, int);
		void osd_stream_position (int);
		void osd_update_status ();
	//Configuration Storage/Retrieval
		const char *config_new_string(const char *, const char *, const char *,
									  const char *, int, xine_config_cb_t,
									  void *);
		int config_new_range(const char *, int, int, int, const char *,
		                     const char *, int, xine_config_cb_t, void *);
		
		int config_new_bool(const char *, int,
							const char *,
		                    const char *,int, xine_config_cb_t, void *);
		
		int config_lookup(const char*, xine_cfg_entry_t *);
		void config_update(xine_cfg_entry_t *);

		void config_save();		
};


#endif
